from charms.reactive import (
    when,
    when_not,
    set_state,
    remove_state
)

from charmhelpers.core.hookenv import (
    config,
    open_port,
    status_set
)
from charmhelpers.core.host import (
    service_restart,
    service_running,
    service_start
)
from charms.layer.nginx import configure_site

from charms.layer.kibana import (
    KIBANA_YML,
    render_conf
)


@when('apt.installed.kibana')
@when_not('kibana.configured')
def write_kibana_conf():
    render_conf(KIBANA_YML, 'kibana.yml.tmpl')
    set_state('kibana.configured')


@when('nginx.available', 'apt.installed.kibana', 'kibana.configured')
@when_not('kibana.nginx.available')
def configure_kibana_nginx():
    status_set('maintenance', 'Configuring Nginx for kibana')

    # Configure nginx for kibana
    configure_site('kibana', 'kibana.conf.tmpl', port=config('port'))

    # Configure nginx for kibana_lb
    configure_site('kibana_proxy', 'kibana_proxy.conf.tmpl')

    status_set('active', 'Nginx configured for kibana frontend')
    set_state('kibana.nginx.available')


@when('nginx.available', 'apt.installed.kibana', 'elasticsearch.available')
#@when_not('kibana.elasticsearch.available')
def elastic_search_available(elasticsearch):
    status_set('maintenance', 'Configuring kibana for elasticsearch')

    remove_state('kibana.elasticsearch.departed')
    remove_state('elasticsearch.departed')

    ES_SERVERS = []
    for unit in elasticsearch.list_unit_data():
        ES_SERVERS.append({'host': unit['host'], 'port': unit['port']})

    # Configure nginx for es_cluster
    configure_site('es_cluster', 'es_cluster.conf.tmpl',
                   es_servers=ES_SERVERS)

    status_set('active', 'Kibana configured for elasticsearch')

    # Set state
    set_state('kibana.elasticsearch.available')


@when('kibana.elasticsearch.available', 'kibana.nginx.available')
@when_not('kibana.started')
def start_kibana():
    open_port(config('port'))
    set_state('kibana.restart')


@when('kibana.restart')
def start_restart_kibana():
    if service_running('kibana'):
        service_restart('kibana')
    else:
        service_start('kibana')

    if service_running('kibana'):
        remove_state('kibana.restart')
    else:
        set_state('kibana.problems')


@when('kibana.problems')
def kibana_problems():
    status_set('blocked', "Kibana problems, please debug.")
    return


@when('website.available')
def configure_website(website):
    website.configure(port=config('port'))


@when('elasticsearch.departed')
@when_not('kibana.elasticsearch.departed')
def set_no_es_block():
    """If no elasticsearch, block and exit
    """
    remove_state('kibana.elasticsearch.configured')
    set_state('kibana.elasticsearch.departed')
